##### Libraries for analyses #####
library(ggplot2)
library(gridExtra)
library(DESeq2)
library(dplyr)
library(tidyr)
library(stringr)
library(readxl)
library(EnhancedVolcano)
library(pcaMethods)
library(RColorBrewer)
library(viridis)
library(tidyverse)

################### RNASeq analysis ################### 
setwd("C:/Users/henri/Dropbox/Zootoca_colouration/05_ENLIGHTEN")
# Set the prefix for each output file name
outputPrefix <- "ventral_DESeq2"

directory <- "C:/Users/henri/Dropbox/Zootoca_colouration/05_ENLIGHTEN"

#make table w/ sample info
sampleFiles<- c("ELT11930.counts",
                "ELT11941.counts",
                "ELT11944.counts",
                "ELT12074.counts",
                "ELT12078.counts")
sampleNames <- c("ELT11930","ELT11941","ELT11944","ELT12074","ELT12078")
sampleCondition <- c("oviparous","oviparous","oviparous","viviparous","viviparous")
sampleTable <- data.frame(sampleName = sampleNames, fileName = sampleFiles, condition = sampleCondition)

treatments = c("viviparous","oviparous")

#make DESeq object
ddsHTSeq <- DESeqDataSetFromHTSeqCount(sampleTable = sampleTable,
                                       directory = directory,
                                       design = ~ condition)

colData(ddsHTSeq)$condition <- factor(colData(ddsHTSeq)$condition,
                                      levels = treatments)

#guts
dds <- DESeq(ddsHTSeq)


#keep only genes with at least 20 counts
dds <- dds[rowSums(counts(dds)) >= 20,]

# Generate results table
DE_table <- results(dds, alpha = 0.05)

# Print summary of DE results
summary(DE_table)

mcols(DE_table, use.names = T)

# Print total number of gene that are significantly DE
length(DE_table$padj) #15800 genes in total
sum(DE_table$padj < 0.05, na.rm = TRUE) #1798 significantly expressed genes

#Arrange by descending p-value and make as data frame
DE_table <- DE_table[order(DE_table$pvalue),]
DE_table <- as.data.frame(DE_table)

#Table of log-transformed counts
log_counts <- as.data.frame(assay(rlog(dds)))

#####PCA#####
library(pcaMethods)
library(ggplot2)
# Run pcaMethods to generate PC scores for expression profiles
Zoo.pca <- pca(t(log_counts), method="svdImpute", nPcs=5, center = TRUE)

# Look at results summary - shows how much variation explained by each PC axes etc.
summary(Zoo.pca)

# Save PC scores for all samples as new data matrix
Zoo.PCs <- scores(Zoo.pca)
Zoo.PCs <- as.data.frame(Zoo.PCs)
Zoo.PCs$condition <- sampleCondition

g <- ggplot(Zoo.PCs,aes(x=PC1,y=PC2, colour = condition)) + 
  geom_point(size=4, alpha = 1)+
  scale_color_manual(name = "Reproductive mode", values=c("#E4C8BF","#DAC654"))+
  scale_x_continuous(limits = c(-50,50)) + scale_y_continuous(limits = c(-50,50))+
  coord_fixed(ratio = 1)+
  labs(x="PC1 (51.79%)", y="PC2 (24.46%)")+
  guides(fill = guide_legend(title.position = "top"))+
  theme_bw() + theme(panel.grid = element_blank(), plot.margin = unit(c(0.3,0,0.3,0), "in"),
                     axis.text = element_text(size = 18, colour = "black"), axis.title = element_text(size = 17), panel.border = element_rect(size = 1.2), 
                     legend.text = element_text(size = 18), legend.title = element_text(size = 17))

# pdf("DESEq_PCA.pdf")
g
# dev.off()

#####DESEq2####
log_counts <- tibble::rownames_to_column(log_counts, "gene")

DE_table <- tibble::rownames_to_column(DE_table, "gene")

#Add gene count info to DE_table to make a master_table
master_table <- inner_join(DE_table,log_counts)
master_table <- master_table %>% mutate(regulation = ifelse(log2FoldChange > 0, "up_in_ovi","up_in_vivi"))

#Table with only the significantly differentially expressed genes
DEG_table <- master_table %>% filter(padj < 0.05)

#write.csv(DEG_table, "DEG_genes.csv")

#### colour pathways ####
pteridine_genes <- c("GCHFR|MYCBP2|PCBD|\\bSPR\\b|TXNDC17|XDH|QDPR")
pteridine_pathway <- master_table %>% filter(str_detect(gene, regex(pteridine_genes, ignore_case = TRUE,))) %>% mutate(link_to_colour = "pteridine pathway")

carotenoid_genes <- c("SCARB1|BCO1|BCO2|DHRS3|DGAT|\\bLRAT\\b|ALDH|\\bADH|\\bRDH|RETSAT")
carotenoid_metabolism <- master_table %>% filter(str_detect(gene, regex(carotenoid_genes, ignore_case = TRUE))) %>% mutate(link_to_colour = "carotenoid metabolism")
carotenoid_metabolism <- carotenoid_metabolism %>% filter(gene != 'ADHFE1')

melanin_genes <- c("\\bTYR\\b|\\bTYRP1\\b|DDT|PMEL|OCA2|SLC24A5|SLC45A2")
melanin_pathway <- master_table %>% filter(str_detect(gene, regex(melanin_genes, ignore_case = TRUE))) %>% mutate(link_to_colour = "melanin pathway")
melanin_pathway$gene <- gsub("SLC45A2", "AIM1", melanin_pathway$gene)

all_pathways <- bind_rows(pteridine_pathway,melanin_pathway,carotenoid_metabolism)
all_pathways <- all_pathways %>% 
  select(log2FoldChange,padj,gene,link_to_colour,regulation,ELT11930,ELT11941,ELT11944,ELT12074,ELT12078) %>%
  gather("sample","log_transformed_counts",6:10) %>%
  mutate(reproductive_mode = ifelse(str_detect(sample,"ELT11"), "oviparous","viviparous")) %>%
  group_by(gene, reproductive_mode,link_to_colour,padj,regulation) %>%
  dplyr::summarise(mean_log_counts = mean(log_transformed_counts))

pteridine_table <- all_pathways %>% filter(link_to_colour == "pteridine pathway") %>% arrange(gene)
carotenoid_table <-all_pathways %>% filter(link_to_colour == "carotenoid metabolism") %>% arrange(gene)
melanin_table <-all_pathways %>% filter(link_to_colour == "melanin pathway") %>% arrange(gene)

melanin_table$gene <- factor(melanin_table$gene, levels = unique(melanin_table$gene))
carotenoid_table$gene <- factor(carotenoid_table$gene, levels = unique(carotenoid_table$gene))
pteridine_table$gene <- factor(pteridine_table$gene, levels = unique(pteridine_table$gene))

### HEATPLOTS ###
library(extrafont)

pteridine.heatmap <- ggplot(data = pteridine_table, 
                            mapping = aes(x = reproductive_mode,
                                          y = gene,
                                          fill = mean_log_counts)) +
  geom_tile(colour = "black", size = 0.75) +
  ylab(label = "Pteridine Synthesis") +
  xlab(label = "") +
  scale_fill_distiller(name = "Mean\nlog2\ncounts", type = "div", palette = "RdYlBu")+
  scale_y_discrete(limits = rev(levels(pteridine_table$gene)))+
  theme(text = element_text(face = "bold", colour = "#000000"),
        panel.background = element_blank(),
        axis.ticks = element_blank(),
        axis.text=element_text(size=14, color = "black"),
        axis.title=element_text(size=22,face="bold"))

pteridine.heatmap

melanin.heatmap <- ggplot(data = melanin_table, 
                          mapping = aes(x = reproductive_mode,
                                        y = gene,
                                        fill = mean_log_counts)) +
  geom_tile(colour = "black", size = 0.7,) +
  coord_fixed(ratio=0.1)+
  ylab(label = "Melanin Synthesis") +
  xlab(label = "") +
  scale_fill_distiller(name = "Mean\nlog2\ncounts", type = "div", palette = "RdYlBu")+
  scale_y_discrete(limits = rev(levels(melanin_table$gene)))+
  theme(text = element_text(face = "bold", colour = "#000000"),
        panel.background = element_blank(),
        axis.ticks = element_blank(),
        axis.text= element_text(size=14, color = "black"),
        axis.text.y = element_text(size=10, color = "black"),
        axis.text.x = element_blank(),
        axis.title=element_text(size=22,face="bold"))#,
#legend.position = "none")


melanin.heatmap

carotenoid.heatmap <- ggplot(data = carotenoid_table, 
                             mapping = aes(x = reproductive_mode,
                                           y = gene,
                                           fill = mean_log_counts)) +
  geom_tile(colour = "black", size = 0.7) +
  coord_fixed(ratio = 0.1) +
  ylab(label = "Carotenoid Metabolism") +
  xlab(label = "") +
  scale_fill_distiller(name = "Mean\nlog2\ncounts", type = "div", palette = "RdYlBu")+
  scale_y_discrete(limits = rev(levels(carotenoid_table$gene)))+
  theme(text = element_text(face = "bold", colour = "#000000"),
        panel.background = element_blank(),
        axis.ticks = element_blank(),
        axis.text= element_text(size=14, color = "black"),
        axis.text.y = element_text(size=10, color = "black"),
        axis.title=element_text(size=22,face="bold"),
        legend.position = "none")

carotenoid.heatmap

# pdf("carotenoid_heatmap.pdf")
# carotenoid.heatmap
# dev.off()
# pdf("melanin_heatmap.pdf")
# melanin.heatmap
# dev.off()
# pdf("pteridine_heatmap.pdf")
# pteridine.heatmap
# dev.off()

#Load tables from Higdon, C. W., Mitra, R. D. & Johnson, S. L. Gene expression analysis of zebrafish melanocytes, iridophores, and retinal pigmented epithelium reveals indicators of biological function and developmental origin. PLoS One 8, e67801 (2013).
iridophores <- read_xlsx("journal.pone.0067801.s017.XLSX",skip = 1)
iridophores_rpe <- read_xlsx("journal.pone.0067801.s015.XLSX",skip = 1)
iridophores_melanocytes <- read_xlsx("journal.pone.0067801.s014.XLSX",skip = 1)

iridophores <- iridophores %>%
  select(1,4)
names(iridophores) <- c("refseq","iridophore")

iridophores_rpe <- iridophores_rpe %>%
  select(1,4)
names(iridophores_rpe) <- c("refseq","iridophore")

iridophores_melanocytes <- iridophores_melanocytes %>%
  select(1,4)
names(iridophores_melanocytes) <- c("refseq","iridophore")

all_iridophores <- rbind(iridophores, iridophores_melanocytes) %>%
  rbind(., iridophores_rpe)

all_iridophores$gene <- gsub(".*\\((.*)\\).*", "\\1", all_iridophores$refseq)
all_iridophores <- all_iridophores %>% select(3)

all_iridophores$gene <- toupper(all_iridophores$gene)
DEG_iridophores = inner_join(DEG_table, all_iridophores)
#write.csv(DEG_iridophores, "DEG_iridophore_genes.csv")

### VOLCANO PLOT ###
volcano_table <- master_table %>% filter(gene != "MATN1")
volcano_table$gene <- as.character(volcano_table$gene)

pathways <- rbind(melanin_pathway,carotenoid_metabolism,pteridine_pathway)
pathways <- pathways %>% filter(padj < 0.05)

pathways$gene <- as.character(pathways$gene)


keyvals <- ifelse(volcano_table$padj >= 0.05, '#D3D3D3', 
                  ifelse(volcano_table$log2FoldChange < 0, '#DAC654',
                         ifelse(volcano_table$log2FoldChange > 0, '#E4C8BF',
                                '#D3D3D3')))
keyvals[is.na(keyvals)] <- '#D3D3D3'
names(keyvals)[keyvals == '#E4C8BF'] <- 'oviparous up'
names(keyvals)[keyvals == '#DAC654'] <- 'viviparous up'
names(keyvals)[keyvals == '#D3D3D3'] <- 'NS'

#pdf("volcano_plot.pdf")
EnhancedVolcano(volcano_table,
                lab = volcano_table$gene,
                selectLab = pathways$gene,
                x = 'log2FoldChange',
                y = 'padj',
                pointSize = 3.0,
                labSize = 4.5,
                labCol = 'black',
                labFace = 'bold',
                boxedLabels = TRUE,
                colAlpha = 1,
                colCustom = keyvals,
                legendPosition = 'bottom',
                legendLabSize = 18,
                legendIconSize = 6.0,
                drawConnectors = TRUE,
                widthConnectors = 1.0,
                lengthConnectors = unit(0.01,"npc"),
                colConnectors = 'black',
                pCutoff = 0.05,
                FCcutoff = 2.0,
                title = NULL,
                subtitle = NULL,
                gridlines.major = FALSE,
                gridlines.minor = FALSE)
#dev.off()

####FIG 4####
#panel a

#DGAT2
dgat2 <- read_xlsx("dgat2_allele_freq_diff.xlsx")
my_threshold <- as.numeric(quantile(dgat2$FREQ_DIFF, 0.975, na.rm = T))
dgat2$outlier <- dgat2$FREQ_DIFF > my_threshold
sum(dgat2$outlier, na.rm = TRUE) #46 outliers


#PMEL
pmel <- read_xlsx("pmel_allele_freq_diff.xlsx")
pmel$FREQ_DIFF <- as.double(pmel$FREQ_DIFF)
my_threshold <- as.numeric(quantile(pmel$FREQ_DIFF, 0.975, na.rm = T))
pmel$outlier <- pmel$FREQ_DIFF > my_threshold
sum(pmel$outlier, na.rm = TRUE) #16 outliers


#BCO2
bco2 <- read_xlsx("bco2_allele_freq_diff.xlsx")
bco2$FREQ_DIFF <- as.double(bco2$FREQ_DIFF)
my_threshold <- as.numeric(quantile(bco2$FREQ_DIFF, 0.975, na.rm = T))
bco2$outlier <- bco2$FREQ_DIFF > my_threshold
sum(bco2$outlier, na.rm = TRUE) #72 outliers

t.test(dgat2$FREQ_DIFF, pmel$FREQ_DIFF, alternative = "two.sided")
t.test(dgat2$FREQ_DIFF, bco2$FREQ_DIFF, alternative = "two.sided")
t.test(pmel$FREQ_DIFF, bco2$FREQ_DIFF, alternative = "two.sided")

#SCARB1
scarb1 <- read_xlsx("scarb1_allele_freq_diff.xlsx")
scarb1$FREQ_DIFF <- as.double(scarb1$FREQ_DIFF)
my_threshold <- as.numeric(quantile(scarb1$FREQ_DIFF, 0.975, na.rm = T))
scarb1$outlier <- scarb1$FREQ_DIFF > my_threshold
sum(scarb1$outlier, na.rm = TRUE) #0 outliers

#TYR
tyr <- read_xlsx("tyr_allele_freq_diff.xlsx")
tyr$FREQ_DIFF <- as.double(tyr$FREQ_DIFF)
my_threshold <- as.numeric(quantile(tyr$FREQ_DIFF, 0.975, na.rm = T))
tyr$outlier <- tyr$FREQ_DIFF > my_threshold
sum(tyr$outlier, na.rm = TRUE) #58 outliers

#TYRP1
tyrp1 <- read_xlsx("tyrp1_allele_freq_diff.xlsx")
tyrp1$FREQ_DIFF <- as.double(tyrp1$FREQ_DIFF)
my_threshold <- as.numeric(quantile(tyrp1$FREQ_DIFF, 0.975, na.rm = T))
tyrp1$outlier <- tyrp1$FREQ_DIFF > my_threshold
sum(tyrp1$outlier, na.rm = TRUE) #17 outliers

#ALDH18A1
aldh18a1 <- read_xlsx("aldh18a1_allele_freq_diff.xlsx")
aldh18a1$FREQ_DIFF <- as.double(aldh18a1$FREQ_DIFF)
my_threshold <- as.numeric(quantile(aldh18a1$FREQ_DIFF, 0.975, na.rm = T))
aldh18a1$outlier <- aldh18a1$FREQ_DIFF > my_threshold
sum(aldh18a1$outlier, na.rm = TRUE) #29 outliers

#DGAT1
dgat1 <- read_xlsx("dgat1_allele_freq_diff.xlsx")
dgat1$FREQ_DIFF <- as.double(dgat1$FREQ_DIFF)
my_threshold <- as.numeric(quantile(dgat1$FREQ_DIFF, 0.975, na.rm = T))
dgat1$outlier <- dgat1$FREQ_DIFF > my_threshold
sum(dgat1$outlier, na.rm = TRUE) #43 outliers

#DHRS3
dhrs3 <- read_xlsx("dhrs3_allele_freq_diff.xlsx")
dhrs3$FREQ_DIFF <- as.double(dhrs3$FREQ_DIFF)
my_threshold <- as.numeric(quantile(dhrs3$FREQ_DIFF, 0.975, na.rm = T))
dhrs3$outlier <- dhrs3$FREQ_DIFF > my_threshold
sum(dhrs3$outlier, na.rm = TRUE) #14 outliers

#SLC24A5
slc24a5 <- read_xlsx("slc24a5_allele_freq_diff.xlsx")
slc24a5$FREQ_DIFF <- as.double(slc24a5$FREQ_DIFF)
my_threshold <- as.numeric(quantile(slc24a5$FREQ_DIFF, 0.975, na.rm = T))
slc24a5$outlier <- slc24a5$FREQ_DIFF > my_threshold
sum(slc24a5$outlier, na.rm = TRUE) #9 outliers

all <- rbind(aldh18a1,dgat1,dgat2,dhrs3,pmel,scarb1,slc24a5,tyr,tyrp1)
plot <- ggplot(all, aes(x = GENE, y = FREQ_DIFF, fill = REGION)) + geom_boxplot() + theme_classic()
plot

# ggsave(filename="allele_freq_differences_boxplot.pdf",
#        plot = plot,
#        device = cairo_pdf,
#        width = 297,
#        height = 210,
#        units = "mm")


#panels b and c
dgat2_windowed100_fst <-read.delim("ovi_vs_vivi_windowed100.weir.fst")
dgat2_windowed100_fst$MEAN_FST <- as.numeric(dgat2_windowed100_fst$MEAN_FST)
dgat2_windowed100_fst <- dgat2_windowed100_fst %>% filter(MEAN_FST > 0)
quantile(dgat2_windowed100_fst$MEAN_FST, c(0.975, 0.995), na.rm = T)
# identify the 95% percentile
my_threshold_dgat2 <- quantile(dgat2_windowed100_fst$MEAN_FST, 0.975, na.rm = T)
# make an outlier column in the data.frame
dgat2_windowed100_fst <- dgat2_windowed100_fst %>% mutate(outlier = ifelse(MEAN_FST > my_threshold_dgat2, "outlier", "background"))
dgat2_windowed100_fst %>% group_by(outlier) %>% tally()

# pdf("dgat2_fst_features.pdf")
ggplot(dgat2_windowed100_fst, aes(BIN_START, MEAN_FST, colour = outlier)) +
  geom_point() +
  theme_classic() + theme(aspect.ratio = 1) +
  scale_colour_manual(name = NULL,values = c("black", "red")) +
  geom_hline(yintercept=my_threshold_dgat2, linetype="dashed", color = "black", size=0.1) +
  xlab("chr5 position") + ylab("Fst") + 
  geom_vline(xintercept = 6104446, size = 0.1) +
  geom_vline(xintercept = 6104561, size = 0.1) +
  geom_vline(xintercept = 6104600, size = 0.1) +
  geom_vline(xintercept = 6128262, size = 0.1) +
  geom_vline(xintercept = 6128381, size = 0.1) +
  geom_vline(xintercept = 6134060, size = 0.1) +
  geom_vline(xintercept = 6134167, size = 0.1) +
  geom_vline(xintercept = 6134700, size = 0.1) +
  geom_vline(xintercept = 6134770, size = 0.1) +
  geom_vline(xintercept = 6139289, size = 0.1) +
  geom_vline(xintercept = 6139493, size = 0.1) +
  geom_vline(xintercept = 6142590, size = 0.1) +
  geom_vline(xintercept = 6142764, size = 0.1) +
  geom_vline(xintercept = 6144011, size = 0.1) +
  geom_vline(xintercept = 6144213, size = 0.1) +
  geom_vline(xintercept = 6146001, size = 0.1) +
  geom_vline(xintercept = 6146155, size = 0.1) +
  geom_vline(xintercept = 6148503, size = 0.1)
# dev.off()

pmel_windowed100_fst <-read.delim("ovi_vs_vivi_windowed100.windowed.weir_pmel.fst")
pmel_windowed100_fst$MEAN_FST <- as.numeric(pmel_windowed100_fst$MEAN_FST)
pmel_windowed100_fst <- pmel_windowed100_fst %>% filter(MEAN_FST > 0)
quantile(pmel_windowed100_fst$MEAN_FST, c(0.975, 0.995), na.rm = T)
# identify outliers
my_threshold_pmel <- quantile(pmel_windowed100_fst$MEAN_FST, 0.975, na.rm = T)
# make an outlier column in the data.frame
pmel_windowed100_fst <- pmel_windowed100_fst %>% mutate(outlier = ifelse(MEAN_FST > my_threshold_pmel, "outlier", "background"))
pmel_windowed100_fst %>% group_by(outlier) %>% tally()

# pdf("pmel_fst_features.pdf")
ggplot(pmel_windowed100_fst, aes(BIN_START, MEAN_FST, colour = outlier)) + 
  geom_point() +
  theme_classic() + theme(aspect.ratio = 1) +
  scale_colour_manual(name = NULL,values = c("black", "red")) + 
  geom_hline(yintercept=my_threshold_pmel, linetype="dashed", color = "black", size=0.1) +
  xlab("chr2 position") + ylab("Fst") + 
  geom_vline(xintercept = 17723922, size = 0.1) +
  geom_vline(xintercept = 17724117, size = 0.1) +
  geom_vline(xintercept = 17724276, size = 0.1) + 	
  geom_vline(xintercept = 17725683, size = 0.1) +
  geom_vline(xintercept = 17725773, size = 0.1) +
  geom_vline(xintercept = 17726123, size = 0.1) +
  geom_vline(xintercept = 17726325, size = 0.1) +
  geom_vline(xintercept = 17727092, size = 0.1) +
  geom_vline(xintercept = 17727185, size = 0.1) +
  geom_vline(xintercept = 17727965, size = 0.1) +
  geom_vline(xintercept = 17728126, size = 0.1) +
  geom_vline(xintercept = 17728831, size = 0.1) +
  geom_vline(xintercept = 17729271, size = 0.1) +
  geom_vline(xintercept = 17730866, size = 0.1) +
  geom_vline(xintercept = 17731312, size = 0.1) +
  geom_vline(xintercept = 17732270, size = 0.1) +
  geom_vline(xintercept = 17732431, size = 0.1) +
  geom_vline(xintercept = 17733628, size = 0.1) +
  geom_vline(xintercept = 17733768, size = 0.1) +
  geom_vline(xintercept = 17734370, size = 0.1) +
  geom_vline(xintercept = 17734516, size = 0.1) +
  geom_vline(xintercept = 17734614, size = 0.1) +
  geom_vline(xintercept = 17734748, size = 0.1) +
  geom_vline(xintercept = 17740438, size = 0.1) +
  geom_vline(xintercept = 17740568, size = 0.1) + 	
  geom_vline(xintercept = 17740504, size = 0.1)
# dev.off()